<?php
/**
 * A Socket registry
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009, 2010 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class SocketRegistry extends BaseRegistry implements Register, RegisterableSocket {
	// Exception constants
	const SOCKET_NOT_REGISTERED = 0xd00;

	/**
	 * Instance of this class
	 */
	private static $registryInstance = null;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Singleton getter for self instance. This class has no factory pattern
	 * because here is no need for special parameters.
	 *
	 * @return	$registryInstance	Instance of this class
	 */
	public final static function createSocketRegistry () {
		// Is an instance there?
		if (is_null(self::$registryInstance)) {
			// Not yet, so create one
			self::$registryInstance = new SocketRegistry();
		} // END - if

		// Return the instance
		return self::$registryInstance;
	}

	/**
	 * Checks wether given socket resource is registered. If $socketResource is
	 * false only the instance will be checked.
	 *
	 * @param	$listenerInstance	An instance of a Listenable class
	 * @param	$socketResource		A valid socket resource
	 * @return	$isRegistered		Wether the given socket resource is registered
	 */
	public function isSocketRegistered (Listenable $listenerInstance, $socketResource) {
		$this->partialStub('Not yet implemented.');
	}

	/**
	 * Registeres given socket for listener or throws an exception if it is already registered
	 *
	 * @param	$listenerInstance	An instance of a Listenable class
	 * @param	$socketResource		A valid socket resource
	 * @throws	SocketAlreadyRegisteredException	If the given socket is already registered
	 * @return	void
	 */
	public function registerSocket (Listenable $listenerInstance, $socketResource) {
		// Is the socket already registered?
		if ($this->isSocketRegistered($listenerInstance, $socketResource)) {
			// Throw the exception
			throw new SocketAlreadyRegisteredException($listenerInstance, BaseListener::EXCEPTION_SOCKET_ALREADY_REGISTERED);
		} // END - if

		$this->partialStub('Not yet implemented.');
	}

	/**
	 * Getter for given listener's socket resource
	 *
	 * @param	$listenerInstance	An instance of a Listenable class
	 * @return	$socketResource		A valid socket resource
	 * @throws	NoSocketRegisteredException		If the requested socket is not registered
	 */
	public function getSocketResource (Listenable $listenerInstance) {
		// The socket must be registered before we can return it
		if (!$this->isSocketRegistered($listenerInstance, false)) {
			// Throw the exception
			throw new NoSocketRegisteredException ($listenerInstance, self::SOCKET_NOT_REGISTERED);
		} // END - if

		$this->partialStub('Not yet implemented.');
	}
}

// [EOF]
?>
